-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}
M.type = "auxilliary"
M.relevantDevice = "transfercase"

local transfercase = nil
local gearbox = nil
--local origViscousCoef = 0
local inputAVSmoother = newExponentialSmoothing(50)
local outputAV1Smoother = newExponentialSmoothing(50)
local outputAV2Smoother = newExponentialSmoothing(50)
local clutchRatioSmoother = newTemporalSmoothingNonLinear(20, 5)

local function updateGFX(dt)

  local inputAV = inputAVSmoother:get(transfercase.inputAV)
  local outputAV1 = outputAV1Smoother:get(transfercase.outputAV1) 
  local outputAV2 = outputAV2Smoother:get(transfercase.outputAV2) 
  local throttle = electrics.values['throttle_input']
  local brake = electrics.values['brake_input']
  local speedPreloadFade = clamp(200 / (outputAV1 + 1e-30),0.25,1)
  
  --local clutchRatio = clamp(0.2 + 0.6 * clamp(throttle * 2 - 0.2,0,1) + 0.6 * clamp(math.abs(outputAV2 - outputAV1) * 0.02 - 0.4,0,1),0,1)
  local clutchRatio = clamp(0.1 + 0.5 * clamp(throttle * 2 - 0.2,0,1) * speedPreloadFade + 0.6 * clamp(math.abs(outputAV2 - outputAV1) * 0.02 - 0.4,0,1),0,1)

  if input.parkingbrake > 0.2 then
    clutchRatio = 0.05
  end 
  
  transfercase.clutchRatio = clutchRatioSmoother:get(clutchRatio, dt)
  --transfercase.lockSpring = 500
  transfercase.lockSpring = 1000
  transfercase.lockTorque = 2000


end

local function init(jbeamData)
  transfercase = powertrain.getDevice(jbeamData.transfercaseName)
  gearbox = powertrain.getDevice(jbeamData.gearboxName or "gearbox")

  if transfercase and gearbox then
    --origViscousCoef = transfercase.viscousCoef
    transfercase.lockSpring = 100
    transfercase.lockTorque = 100
    M.updateGFX = updateGFX
  end
end

M.init = init
M.reset = nop
M.updateGFX = nop

return M